@@ -7,8 +7,8 @@ from keg_elements.forms import Form, ModelForm, FieldMeta, MultiCheckboxField
 from keg_elements.forms.validators import ValidateUnique
 from sqlalchemy.sql.functions import coalesce
 from sqlalchemy_utils import EmailType
-from webhelpers2.html.tags import link_to
 from wtforms.fields import (
+    BooleanField,
     DateField,
     HiddenField,
     PasswordField,
@@ -18,10 +18,12 @@ from wtforms import ValidationError, validators
 from wtforms_components.widgets import EmailInput
 
 from keg_auth.extensions import lazy_gettext as _
+from keg_auth.libs.templates import link_to
 from keg_auth.model import get_username_key
 
 
 def login_form():
+    """Returns a Login form class that handles username options."""
     login_id_label = _(u'User ID')
     login_id_validators = [validators.DataRequired()]
 
@@ -41,6 +43,7 @@ def login_form():
 
 
 class ForgotPassword(Form):
+    """Returns a form to capture email for password reset."""
     email = StringField(_(u'Email'), validators=[
         validators.DataRequired(),
         validators.Email(),
@@ -48,6 +51,7 @@ class ForgotPassword(Form):
 
 
 class SetPassword(Form):
+    """Returns a form to capture password/confirmation and apply password policy."""
     password = PasswordField(_('New Password'), validators=[
         validators.DataRequired(),
         validators.EqualTo('confirm', message=_('Passwords must match'))
@@ -142,6 +146,9 @@ class _ValidatePasswordRequired(object):
 
 def user_form(config=None, allow_superuser=False, endpoint='',
               fields=['is_enabled', 'disabled_utc']):
+    """Returns a form for User CRUD.
+
+    Form is customized depending on the fields and superuser setting passed in."""
     config = config or {}
     user_cls = flask.current_app.auth_manager.entity_registry.user_cls
 
@@ -200,6 +207,10 @@ def user_form(config=None, allow_superuser=False, endpoint='',
             ])
             confirm = PasswordField(_('Confirm Password'))
             field_order = field_order + ('reset_password', 'confirm')
+        else:
+            # place a Send Welcome field after the initial set of fields
+            send_welcome = BooleanField('Send Welcome Email', default=True)
+            field_order = tuple(_fields + ['send_welcome'] + list(field_order[len(_fields):]))
 
         def get_object_by_field(self, field):
             return user_cls.get_by(username=field.data)
@@ -212,10 +223,18 @@ def user_form(config=None, allow_superuser=False, endpoint='',
             order = ('csrf_token', ) + self.field_order
             return (getattr(self, field_id) for field_id in order)
 
+        def after_init(self, args, kwargs):
+            if kwargs.get('obj') and hasattr(self, 'send_welcome'):
+                self.send_welcome = None
+                self.field_order = tuple(filter(lambda v: v != 'send_welcome', self.field_order))
+
+            return super().after_init(args, kwargs)
+
     return User
 
 
 def group_form(endpoint):
+    """Returns a form for Group CRUD."""
     group_cls = flask.current_app.auth_manager.entity_registry.group_cls
 
     def html_link(obj):
@@ -242,6 +261,7 @@ def group_form(endpoint):
 
 
 def bundle_form(endpoint):
+    """Returns a form for Bundle CRUD."""
     bundle_cls = flask.current_app.auth_manager.entity_registry.bundle_cls
 
     def html_link(obj):
