@@ -59,6 +59,7 @@ class KAPasswordType(PasswordType):
 
 
 class UserMixin(object):
+    """Generic mixin for user entities."""
     # These two attributes are needed by Flask-Login.
     is_anonymous = False
     is_authenticated = True
@@ -80,6 +81,10 @@ class UserMixin(object):
     # when a user logs in.
     last_login_utc = sa.Column(ArrowType, nullable=True, default=None, server_default=None)
 
+    # The datetime when a user will be disabled. User will be inactive if this is set
+    # to a datetime in the past.
+    disabled_utc = sa.Column(ArrowType, nullable=True, default=None, server_default=sa.null())
+
     # is_active defines the complexities of how to determine what users are active. For instance,
     #   if email is in scope, we need to have an additional flag to verify users, and that would
     #   get included in is_active logic.
@@ -105,10 +110,6 @@ class UserMixin(object):
         )
         return sa_sql.case([(is_disabled_expr, sa.true())], else_=sa.false())
 
-    # The datetime when a user will be disabled. User will be inactive if this is set
-    # to a datetime in the past.
-    disabled_utc = sa.Column(ArrowType, nullable=True, default=None, server_default=sa.null())
-
     def get_id(self):
         # Flask-Login requires that this return a string value for the session
         return str(self.session_key)
@@ -273,6 +274,7 @@ class UserMixin(object):
             * expiration
             * password was not already reset since token was generated
             * user has not signed in since token was generated
+
         :param token: string representation of token to verify
         :return: bool indicating token validity
         """
@@ -311,7 +313,7 @@ class UserMixin(object):
 
 
 class UserTokenMixin(object):
-
+    """Mixin for users who will be authenticated by tokens."""
     token = sa.Column(KAPasswordType(onload=_create_cryptcontext_kwargs))
 
     @classmethod
@@ -375,6 +377,7 @@ class UserTokenMixin(object):
 
 
 class UserEmailMixin(object):
+    """Mixin for users who will be authenticated by email/password."""
     # Assume the user will need to verify their email address before they become active.
     is_verified = sa.Column(sa.Boolean, nullable=False, default=False)
     email = sa.Column(EmailType, nullable=False, unique=True)
@@ -423,6 +426,7 @@ class UserEmailMixin(object):
 
 
 class PermissionMixin(object):
+    """Generic mixin for permissions."""
     token = sa.Column(sa.Unicode(1024), nullable=False, unique=True)
     description = sa.Column(sa.Unicode)
 
@@ -439,24 +443,19 @@ class PermissionMixin(object):
 
 
 class BundleMixin(object):
+    """Generic mixin for permission bundles."""
     name = sa.Column(sa.Unicode(1024), nullable=False, unique=True)
 
     @might_commit
     @might_flush
     @classmethod
     def edit(cls, oid=None, **kwargs):
-        """obj = cls.query.get(oid)
-
-        rights_keys = ('permissions', )
-        original_rights = _rights_as_dict(obj, *rights_keys)
-        original_users = {user.id for user in obj.users}
-        original_groups = {group.id for group in obj.groups}"""
-
         obj = super(BundleMixin, cls).edit(oid=oid, _commit=False, **kwargs)
         return obj
 
 
 class GroupMixin(object):
+    """Generic mixin for user groups."""
     name = sa.Column(sa.Unicode(1024), nullable=False, unique=True)
 
     def get_all_permissions(self):
@@ -498,6 +497,7 @@ class GroupMixin(object):
 
 
 class AttemptMixin(object):
+    """Generic mixin for logging user login attempts."""
     # Form input data, e.g. username
     user_input = sa.Column(sa.Unicode(512), nullable=False)
 
@@ -527,6 +527,7 @@ class AttemptMixin(object):
 
 
 def get_username(user):
+    """Based on the registered user entity, find the column representing the login ID."""
     user_cls = registry().get_entity_cls('user')
     return getattr(user, get_username_key(user_cls))
 
