@@ -1,10 +1,11 @@
+from unittest import mock
+
 from blazeutils.datastructures import BlankObject
 import flask
 import freezegun
 import arrow
 import flask_webtest
 from keg.db import db
-import mock
 import pytest
 import sqlalchemy as sa
 from werkzeug.datastructures import MultiDict
@@ -299,6 +300,13 @@ class TestViews(object):
         assert doc('div#page-content a').text() == 'Cancel'
         assert doc('div#page-content a').attr('href') == '/login'
 
+    def test_reset_pw_head(self):
+        user = ents.User.testing_create()
+        token = user.token_generate()
+
+        client = flask_webtest.TestApp(flask.current_app)
+        client.head('/reset-password/{}/{}'.format(user.id, token), status=405)
+
     def test_reset_pw_actions(self):
         user = ents.User.testing_create()
         token = user.token_generate()
@@ -511,6 +519,11 @@ class TestPermissionsRequired:
         client.get('/protected-class', status=405)
         client.get('/protected-class2', status=302)
 
+    def test_user_requirement_excludes_method(self):
+        client = flask_webtest.TestApp(flask.current_app)
+        # should have a 200 response
+        client.options('/protected-class2')
+
 
 class TestRequestLoaders(object):
     def test_token_auth_no_token(self):
@@ -565,6 +578,19 @@ class TestUserCrud(ViewTestBase):
         assert user.groups == [group_approve]
         assert user.bundles == [bundle_approve]
 
+    def test_add_no_welcome(self):
+        resp = self.client.get('/users/add')
+
+        resp.form['email'] = 'abc3@example.com'
+        resp.form['send_welcome'] = False
+        with mail_ext.record_messages() as outbox:
+            resp = resp.form.submit()
+        assert resp.status_code == 302
+        assert resp.location.endswith('/users')
+        assert resp.flashes == [('success', 'Successfully created User')]
+        assert len(outbox) == 0
+        assert self.user_ent.get_by(email='abc3@example.com')
+
     def test_resend_verification(self):
         self.current_user.is_verified = True
         self.current_user.permissions = ents.Permission.query.filter_by(token='auth-manage').all()
@@ -716,6 +742,12 @@ class TestUserCrud(ViewTestBase):
         assert resp.status_code == 302
         assert resp.location.endswith('/users?session_key=foo')
 
+    @mock.patch('flask.render_template', autospec=True, spec_set=True)
+    def test_add_template_args(self, m_render):
+        m_render.return_value = 'foo'
+        self.client.get('/users/add')
+        assert 'a_template_variable' in m_render.call_args[1]
+
     def test_edit(self):
         user_edit = ents.User.testing_create(
             groups=[ents.Group.testing_create()],
@@ -744,6 +776,11 @@ class TestUserCrud(ViewTestBase):
         assert resp.flashes == [('success', 'Successfully modified User')]
         assert self.user_ent.get_by(email='foo@bar.baz')
 
+    def test_edit_disallowed_by_permission(self):
+        user = ents.User.testing_create()
+        client = AuthTestApp(flask.current_app, user=user)
+        client.get('/users/99999/edit', status=403)
+
     def test_edit_with_session_key(self):
         user_edit = ents.User.testing_create()
 
@@ -841,6 +878,11 @@ class TestUserCrud(ViewTestBase):
 
         self.client.get('/users/{}/delete'.format(user_delete_id), status=302)
 
+    def test_delete_disallowed_by_permission(self):
+        user = ents.User.testing_create()
+        client = AuthTestApp(flask.current_app, user=user)
+        client.get('/users/99999/delete', status=403)
+
     def test_delete_with_session_key(self):
         user_delete = ents.User.testing_create()
 
@@ -879,6 +921,12 @@ class TestUserCrud(ViewTestBase):
         assert resp.pyquery('#page-content')('h1').eq(0).text() == 'Users'
         assert resp.pyquery('.grid-header-add-link').eq(0).text() == 'Create User'
 
+    @mock.patch('flask.render_template', autospec=True, spec_set=True)
+    def test_list_template_args(self, m_render):
+        m_render.return_value = 'foo'
+        self.client.get('/users')
+        assert 'a_template_variable' in m_render.call_args[1]
+
     def test_list_alternate_ident_field(self):
         # with the mock here, authentication/authorization will also be happening on the alternate
         # class, which doesn't have relationships like permissions. So, to make this easy, make it
